Package: logging
Version: 0.10-111
Title: R Logging Package
Authors@R: c(
    person("Mario", "Frasca", email = "mariotomo@gmail.com", role = c("aut", "cre")),
    person("Walerian", "Sokolowski", email = "r-logging@wlogsolutions.com", role = c("aut"))
    )
Description: Pure R implementation of the ubiquitous log4j package. It offers hierarchic 
  loggers, multiple handlers per logger, level based filtering, space handling in messages 
  and custom formatting.
URL: https://github.com/WLOGSolutions/r-logging
BugReports: https://github.com/WLOGSolutions/r-logging/issues
License: GPL-3
Encoding: UTF-8
Language: en-US
Depends: R (>= 3.2.0)
Imports: methods
Suggests: testthat, crayon
RoxygenNote: 6.1.1
NeedsCompilation: no
Packaged: 2026-05-21 14:33:31 UTC; mario
Author: Mario Frasca [aut, cre],
  Walerian Sokolowski [aut]
Maintainer: Mario Frasca <mariotomo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 21:50:30 UTC
Built: R 4.5.2; ; 2026-05-22 05:38:27 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(updateOptions,Logger)
S3method(updateOptions,character)
S3method(updateOptions,environment)
export(addHandler)
export(basicConfig)
export(getHandler)
export(getLogger)
export(levellog)
export(logReset)
export(logdebug)
export(logerror)
export(logfine)
export(logfiner)
export(logfinest)
export(loginfo)
export(loglevels)
export(logwarn)
export(removeHandler)
export(resetMsgComposer)
export(setLevel)
export(setMsgComposer)
export(updateOptions)
export(writeToConsole)
export(writeToFile)
import(methods)

# linux-system-roles/logging
![CI Testing](https://github.com/linux-system-roles/logging/workflows/tox/badge.svg)

## Table of Contents

<!--ts-->
  * [Background](#background)
  * [Definitions](#definitions)
  * [Logging Configuration Overview](#logging-configuration-overview)
  * [Variables](#variables)
    * [Logging_inputs options](#logging_inputs-options)
    * [Logging_outputs options](#logging_outputs-options)
    * [Logging_flows options](#logging_flows-options)
    * [Security options](#security-options)
    * [Server performance optimization options](#server-performance-optimization-options)
    * [Other options](#other-options)
  * [Update and Delete](#update-and-delete)
  * [Configuration Examples](#configuration-examples)
    * [Standalone configuration](#standalone-configuration)
    * [Client configuration](#client-configuration)
    * [Server configuration](#server-configuration)
    * [Client configuration with Relp](#client-configuration-with-relp)
    * [Server configuration with Relp](#server-configuration-with-relp)
  * [Port and SELinux](#port-and-selinux)
  * [Providers](#providers)
  * [Tests](#tests)
<!--te-->

## Background

Logging role is an abstract layer for provisioning and configuring the logging system. Currently, rsyslog is the only supported provider.

In the nature of logging, there are multiple ways to read logs and multiple ways to output them. For instance, the logging system may read logs from local files, or read them from systemd/journal, or receive them from the other logging system over the network. Then, the logs may be stored in the local files in the /var/log directory, or sent to Elasticsearch, or forwarded to other logging system. The combination between the inputs and the outputs needs to be flexible. For instance, you may want to inputs from journal stored just in the local file, while inputs read from files stored in the local log files as well as forwarded to the other logging system.

To satisfy such requirements, logging role introduced 3 primary variables `logging_inputs`, `logging_outputs`, and `logging_flows`. The inputs are represented in the list of `logging_inputs` dictionary, the outputs are in the list of `logging_outputs` dictionary, and the relationship between them are defined as a list of `logging_flows` dictionary. The details are described in [Logging Configuration Overview](#logging-configuration-overview).

## Requirements

This role is supported on RHEL-7+, CentOS Stream-8+ and Fedora distributions.

The role requires the `firewall` role and the `selinux` role from the
`fedora.linux_system_roles` collection, if `logging_manage_firewall`
and `logging_manage_selinux` is set to true, respectively.
(Please see also the variables in the [`Other options`](#other-options) section.)

If the `logging` is a role from the `fedora.linux_system_roles`
collection or from the Fedora RPM package, the requirement is already
satisfied.

Otherwise, please run the following command line to install the collection.
```
ansible-galaxy collection install -r meta/collection-requirements.yml
```

## Definitions

- `logging_inputs` - List of logging inputs dictionary to specify input types.
    * `basics` - basic inputs configuring inputs from systemd journal or unix socket.
    * `files` - files inputs configuring inputs from local files.
    * `remote` - remote inputs configuring inputs from the other logging system over network.
    * `ovirt` - ovirt inputs configuring inputs from the oVirt system.
- `logging_outputs` - List of logging outputs dictionary to specify output types.
    * `elasticsearch` - elasticsearch outputs configuring outputs to elasticsearch. It is available only when the input is `ovirt`.
    * `files` - files outputs configuring outputs to the local files.
    * `forwards` - forwards outputs configuring outputs to the other logging system.
    * `remote_files` - remote files outputs configuring outputs from the other logging system to the local files.
- `logging_flows` - List of logging flows dictionary to define relationships between logging inputs and outputs.
- [`Rsyslog`](https://www.rsyslog.com/) - The logging role default log provider used for log processing.
- [`Elasticsearch`](https://www.elastic.co/) - Elasticsearch is a distributed, search and analytic engine for all types of data. One of the supported outputs in the logging role.

## Logging Configuration Overview

Logging role allows to have variables `logging_inputs`, `logging_outputs`, and `logging_flows` with additional options to configure logging system such as `rsyslog`.

Currently, the logging role supports four types of logging inputs: `basics`, `files`, `ovirt`, and `remote`.  And four types of outputs: `elasticsearch`, `files`, `forwards`, and `remote_files`.  To deploy configuration files with these inputs and outputs, specify the inputs as `logging_inputs` and the outputs as `logging_outputs`. To define the flows from inputs to outputs, use `logging_flows`.  The `logging_flows` has three keys `name`, `inputs`, and `outputs`, where `inputs` is a list of `logging_inputs name` values and `outputs` is a list of `logging_outputs name` values.

This is a schematic logging configuration to show log messages from input_nameA are passed to output_name0 and output_name1; log messages from input_nameB are passed only to output_name1.

```yaml
---
- name: a schematic logging configuration
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: input_nameA
        type: input_typeA
      - name: input_nameB
        type: input_typeB
    logging_outputs:
      - name: output_name0
        type: output_type0
      - name: output_name1
        type: output_type1
    logging_flows:
      - name: flow_nameX
        inputs: [input_nameA]
        outputs: [output_name0, output_name1]
      - name: flow_nameY
        inputs: [input_nameB]
        outputs: [output_name1]
```

## Variables

### Logging_inputs options

`logging_inputs`: A list of the following dictionaries to configure inputs.

#### common keys

- `name`: Unique name of the input. Used in the `logging_flows` inputs list and a part of the generated config filename.
- `type`: Type of the input element. Currently, `basics`, `files`, `ovirt`, and `remote` are supported. The `type` is used to specify a task type which corresponds to a directory name in roles/rsyslog/{tasks,vars}/inputs/.
- `state`: State of the configuration file. `present` or `absent`. Default to `present`.

#### basics type

`basics` input supports reading logs from systemd journal or systemd unix socket.

Available options:
- `kernel_message`: Load `imklog` if set to `true`. Default to `false`.
- `use_imuxsock`: Use `imuxsock` instead of `imjournal`. Default to `false`.
- `ratelimit_burst`: Maximum number of messages that can be emitted within ratelimit_interval. Default to `20000` if use_imuxsock is false. Default to `200` if use_imuxsock is true.
- `ratelimit_interval`: Interval to evaluate ratelimit_burst. Default to `600` seconds if use_imuxsock is false. Default to `0` if use_imuxsock is true. 0 indicates ratelimiting is turned off.
- `persist_state_interval`: Journal state is persisted every value messages. Default to `10`. Effective only when use_imuxsock is false.

#### files type

`files` input supports reading logs from the local files.

Available options:
- `input_log_path`: File name to be read by the imfile plugin. The value should be full path. Wildcard '\*' is allowed in the path.  Default to `/var/log/containers/*.log`.
`facility`: Facility to filter the inputs from the files.
`severity`: Severity to filter the inputs from the files.
`startmsg_regex`: The regular expression that matches the start part of a message.
`endmsg_regex`: The regular expression that matches the last part of a message.

#### ovirt type

`ovirt` input supports oVirt specific inputs.

Available options:
- `subtype`: ovirt input subtype. Value is one of `engine`, `collectd`, and `vdsm`.
- `ovirt_env_name`: ovirt environment name. Default to `engine`.
- `ovirt_env_uuid`: ovirt uuid. Default to none.

Available options for engine and vdsm:
- `ovirt_elasticsearch_index_prefix`: Index prefix for elasticsearch. Default to `project.ovirt-logs`.
- `ovirt_engine_fqdn`: ovirt engine fqdn. Default to none.
- `ovirt_input_file`: ovirt input file. Default to `/var/log/ovirt-engine/test-engine.log` for `engine`; default to `/var/log/vdsm/vdsm.log` for `vdsm`.
- `ovirt_vds_cluster_name`: vds cluster name. Default to none.

Available options for collectd:
- `ovirt_collectd_port`: collectd port number. Default to `44514`.
- `ovirt_elasticsearch_index_prefix`: Index prefix for elasticsearch. Default to `project.ovirt-metrics`.

#### relp type

`relp` input supports receiving logs from the remote logging system over the network using relp.

Available options:
- `port`: Port number Relp is listening to. Default to `20514`. See also [Port and SELinux](#port-and-selinux).
- `tls`: If true, encrypt the connection with TLS. You must provide key/certificates and triplets {`ca_cert`, `cert`, `private_key`} and/or {`ca_cert_src`, `cert_src`, `private_key_src`}. Default to `true`.
- `ca_cert`: Path to CA cert to configure Relp with tls. Default to `/etc/pki/tls/certs/basename of ca_cert_src`.
- `cert`: Path to cert to configure Relp with tls. Default to `/etc/pki/tls/certs/basename of cert_src`.
- `private_key`: Path to key to configure Relp with tls. Default to `/etc/pki/tls/private/basename of private_key_src`.
- `ca_cert_src`: Local CA cert file path which is copied to the target host. If `ca_cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `cert_src`: Local cert file path which is copied to the target host. If `cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `private_key_src`: Local key file path which is copied to the target host. If `private_key` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `pki_authmode`: Specifying the authentication mode. `name` or `fingerprint` is accepted. Default to `name`.
- `permitted_clients`: List of hostnames, IP addresses, fingerprints(sha1), and wildcard DNS domains which will be allowed by the `logging` server to connect and send logs over TLS. Default to `['*.{{ logging_domain }}']`

#### remote type

`remote` input supports receiving logs from the remote logging system over the network.

Available options:
- `udp_ports`: List of UDP port numbers to listen. If set, the `remote` input listens on the UDP ports. No defaults. If both `udp_ports` and `tcp_ports` are set in a `remote` input item, `udp_ports` is used and `tcp_ports` is dropped. See also [Port and SELinux](#port-and-selinux).
- `tcp_ports`: List of TCP port numbers to listen. If set, the `remote` input listens on the TCP ports. Default to `[514]`. If both `udp_ports` and `tcp_ports` are set in a `remote` input item, `udp_ports` is used and `tcp_ports` is dropped. If both `udp_ports` and `tcp_ports` are not set in a `remote` input item, `tcp_ports: [514]` is added to the item. See also [Port and SELinux](#port-and-selinux).
- `tls`: Set to `true` to encrypt the connection using the default TLS implementation used by the provider. Default to `false`.
- `pki_authmode`: Specifying the default network driver authentication mode. `x509/name`, `x509/fingerprint`, or `anon` is accepted. Default to `x509/name`.
- `permitted_clients`: List of hostnames, IP addresses, fingerprints(sha1), and wildcard DNS domains which will be allowed by the `logging` server to connect and send logs over TLS. Default to `['*.{{ logging_domain }}']`

**Note:** There are 3 types of items in the remote type - udp, plain tcp and tls tcp. The udp type configured using `udp_ports`; the plain tcp type is configured using `tcp_ports` without `tls` or with `tls: false`; the tls tcp type is configured using `tcp_ports` with `tls: true` at the same time. Please note there might be only one instance of each of the three types. E.g., if there are 2 `udp` type items, it fails to deploy.

```yaml
  # Valid configuration example
  - name: remote_udp
    type: remote
    udp_ports: [514, ...]
  - name: remote_ptcp
    type: remote
    tcp_ports: [514, ...]
  - name: remote_tcp
    type: remote
    tcp_ports: [6514, ...]
    tls: true
    pki_authmode: x509/name
    permitted_clients: ['*.example.com']
```

```yaml
  # Invalid configuration example 1; duplicated udp
  - name: remote_udp0
    type: remote
    udp_ports: [514]
  - name: remote_udp1
    type: remote
    udp_ports: [1514]
```

```yaml
  # Invalid configuration example 2; duplicated tcp
  - name: remote_implicit_tcp
    type: remote
  - name: remote_tcp
    type: remote
    tcp_ports: [1514]
```

### Logging_outputs options

`logging_outputs`: A list of following dictionary to configure outputs.

#### common keys

- `name`: Unique name of the output. Used in the `logging_flows` outputs list and a part of the generated config filename.
- `type`: Type of the output element. Currently, `elasticsearch`, `files`, `forwards`, and `remote_files` are supported. The `type` is used to specify a task type which corresponds to a directory name in roles/rsyslog/{tasks,vars}/outputs/.
- `state`: State of the configuration file. `present` or `absent`. Default to `present`.

#### elasticsearch type

`elasticsearch` output supports sending logs to Elasticsearch. It is available only when the input is `ovirt`. Assuming Elasticsearch is already configured and running.

Available options:
- `server_host`: Host name Elasticsearch is running on. The value is a single host or list of hosts. **Required**.
- `server_port`: Port number Elasticsearch is listening to. Default to `9200`.
- `index_prefix`: Elasticsearch index prefix the particular log will be indexed to. **Required**.
- `input_type`: Specifying the input type. Currently only type `ovirt` is supported. Default to `ovirt`.
- `retryfailures`: Specifying whether retries or not in case of failure. Allowed value is `true` or `false`. Default to `true`.
- `tls`: If true, encrypt the connection with TLS. You must provide key/certificates and triplets {`ca_cert`, `cert`, `private_key`} and/or {`ca_cert_src`, `cert_src`, `private_key_src`}. Default to `true`.
- `use_cert`: [DEPRECATED] If true, encrypt the connection with TLS. You must provide key/certificates and triplets {`ca_cert`, `cert`, `private_key`} and/or {`ca_cert_src`, `cert_src`, `private_key_src`}. Default to `true`. Option `use_cert` is deprecated in favor of `tls` and `use_cert` will be removed in the next minor release.
- `ca_cert`: Path to CA cert for Elasticsearch. Default to `/etc/pki/tls/certs/basename of ca_cert_src`.
- `cert`: Path to cert to connect to Elasticsearch. Default to `/etc/pki/tls/certs/basename of cert_src`.
- `private_key`: Path to key to connect to Elasticsearch. Default to `/etc/pki/tls/private/basename of private_key_src`.
- `ca_cert_src`: Local CA cert file path which is copied to the target host. If `ca_cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `cert_src`: Local cert file path which is copied to the target host. If `cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `private_key_src`: Local key file path which is copied to the target host. If `private_key` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `uid`: If basic HTTP authentication is deployed, the user name is specified with this key.

logging_elasticsearch_password: If basic HTTP authentication is deployed, the password is specified with this global variable. Please be careful that this `logging_elasticsearch_password` is a global variable to be placed at the same level as `logging_output`, `logging_input`, and `logging_flows` are. Another things to be aware of are this `logging_elasticsearch_password` is shared among all the elasticsearch outputs. That is, the elasticsearch servers should share one password if there are multiple of servers. Plus, the uid and password are configured if both of them are found in the playbook. For instance, if there are multiple elasticsearch outputs and one of them is missing the `uid` key, then the configured output does not have the uid and password.

#### files type

`files` output supports storing logs in the local files usually in /var/log.

Available options:
- `facility`: Facility in selector; default to `*`.
- `severity`: Severity in selector; default to `*`.
- `exclude`: Exclude list used in selector; default to none.
- `property`: Property in property-based filter; no default
- `property_op`: Operation in property-based filter; In case of not `!`, put the `property_op` value in quotes; default to `contains`
- `property_value`: Value in property-based filter; default to `error`
- `path`: Path to the output file.

logging_files_template_format: Set default template for the files output.
Allowed values are `traditional`, `syslog`, and `modern`. Default to `modern`.

**Note:** Selector options and property-based filter options are exclusive. If Property-based filter options are defined, selector options will be ignored.

**Note:** Unless the above options are given, these local file outputs are configured.

```
  kern.*                                      /dev/console
  *.info;mail.none;authpriv.none;cron.none    /var/log/messages
  authpriv.*                                  /var/log/secure
  mail.*                                      -/var/log/maillog
  cron.*                                      -/var/log/cron
  *.emerg                                     :omusrmsg:*
  uucp,news.crit                              /var/log/spooler
  local7.*
```

#### forwards type

`forwards` output sends logs to the remote logging system over the network.

Available options:
- `facility`: Facility in selector; default to `*`.
- `severity`: Severity in selector; default to `*`.
- `exclude`: Exclude list used in selector; default to none.
- `property`: Property in property-based filter; no default
- `property_op`: Operation in property-based filter; In case of not `!`, put the `property_op` value in quotes; default to `contains`
- `property_value`: Value in property-based filter; default to `error`
- `target`: Target host (fqdn). **Required**.
- `udp_port`: UDP port number. Default to `514`.
- `tcp_port`: TCP port number. Default to `514`.
- `tls`: Set to `true` to encrypt the connection using the default TLS implementation used by the provider. Default to `false`.
- `pki_authmode`: Specifying the default network driver authentication mode. `x509/name`, `x509/fingerprint`, or `anon` is accepted. Default to `x509/name`.
- `permitted_server`: Hostname, IP address, fingerprint(sha1) or wildcard DNS domain of the server which this client will be allowed to connect and send logs over TLS. Default to `*.{{ logging_domain }}`
- `template`: Template format for the particular forwards output. Allowed values are `traditional`, `syslog`, and `modern`. Default to `modern`.

logging_forwards_template_format: Set default template for the forwards output.
Allowed values are `traditional`, `syslog`, and `modern`. Default to `modern`.

**Note:** Selector options and property-based filter options are exclusive. If Property-based filter options are defined, selector options will be ignored.

#### relp type

`relp` output sends logs to the remote logging system over the network using relp.

Available options:
- `target`: Host name the remote logging system is running on. **Required**.
- `port`: Port number the remote logging system is listening to. Default to `20514`.
- `tls`: If true, encrypt the connection with TLS. You must provide key/certificates and triplets {`ca_cert`, `cert`, `private_key`} and/or {`ca_cert_src`, `cert_src`, `private_key_src`}. Default to `true`.
- `ca_cert`: Path to CA cert to configure Relp with tls. Default to `/etc/pki/tls/certs/basename of ca_cert_src`.
- `cert`: Path to cert to configure Relp with tls. Default to `/etc/pki/tls/certs/basename of cert_src`.
- `private_key`: Path to key to configure Relp with tls. Default to `/etc/pki/tls/private/basename of private_key_src`.
- `ca_cert_src`: Local CA cert file path which is copied to the target host. If `ca_cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `cert_src`: Local cert file path which is copied to the target host. If `cert` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `private_key_src`: Local key file path which is copied to the target host. If `private_key` is specified, it is copied to the location. Otherwise, to logging_config_dir.
- `pki_authmode`: Specifying the authentication mode. `name` or `fingerprint` is accepted. Default to `name`.
- `permitted_servers`: List of hostnames, IP addresses, fingerprints(sha1), and wildcard DNS domains which will be allowed by the `logging` client to connect and send logs over TLS. Default to `['*.{{ logging_domain }}']`

#### remote_files type

`remote_files` output stores logs to the local files per remote host and program name originated the logs.

Available options:
- `facility`: Facility in selector; default to `*`.
- `severity`: Severity in selector; default to `*`.
- `exclude`: Exclude list used in selector; default to none.
- `property`: Property in property-based filter; no default
- `property_op`: Operation in property-based filter; In case of not `!`, put the `property_op` value in quotes; default to `contains`
- `property_value`: Value in property-based filter; default to `error`
- `async_writing`: If set to `true`, the files are written asynchronously. Allowed value is `true` or `false`. Default to `false`.
- `client_count`: Count of client logging system supported this rsyslog server. Default to `10`.
- `io_buffer_size`: Buffer size used to write output data. Default to `65536` bytes.
- `remote_log_path`: Full path to store the filtered logs.
                       This is an example to support the per host output log files
                       `/path/to/output/dir/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log`
- `remote_sub_path`: Relative path to logging_system_log_dir to store the filtered logs.

**Note:** Selector options and property-based filter options are exclusive. If Property-based filter options are defined, selector options will be ignored.

**Note:** If both `remote_log_path` and `remote_sub_path` are _not_ specified, the remote_file output configured with the following settings.

```
  template(
    name="RemoteMessage"
    type="string"
    string="/var/log/remote/msg/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log"
  )
  template(
    name="RemoteHostAuthLog"
    type="string"
    string="/var/log/remote/auth/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log"
  )
  template(
    name="RemoteHostCronLog"
    type="string"
    string="/var/log/remote/cron/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log"
  )
  template(
    name="RemoteHostMailLog"
    type="string"
    string="/var/log/remote/mail/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log"
  )
  ruleset(name="unique_remote_files_output_name") {
    authpriv.*   action(name="remote_authpriv_host_log" type="omfile" DynaFile="RemoteHostAuthLog")
    *.info;mail.none;authpriv.none;cron.none action(name="remote_message" type="omfile" DynaFile="RemoteMessage")
    cron.*       action(name="remote_cron_log" type="omfile" DynaFile="RemoteHostCronLog")
    mail.*       action(name="remote_mail_service_log" type="omfile" DynaFile="RemoteHostMailLog")
  }
```

### Logging_flows options

- `name`: Unique name of the flow.
- `inputs`: A list of inputs, from which processing log messages starts.
- `outputs`: A list of outputs. to which the log messages are sent.

### Security options

These variables are set in the same level of the `logging_inputs`, `logging_output`, and `logging_flows`.

#### logging_pki_files

Specifying either of the paths of the ca_cert, cert, and key on the control host or
the paths of theirs on the managed host or both of them.
When TLS connection is configured, `ca_cert_src` and/or `ca_cert` is required.
To configure the certificate of the logging system, `cert_src` and/or `cert` is required.
To configure the private key of the logging system, `private_key_src` and/or `private_key` is required.

```
  ca_cert_src:     location of the ca_cert on the control host; if given, the file is copied to the managed host.
  cert_src:        location of the cert on the control host; if given, the file is copied to the managed host.
  private_key_src: location of the key on the control host; if given, the file is copied to the managed host.
  ca_cert:     path to be deployed on the managed host; the path is also used in the rsyslog config.
               default to /etc/pki/tls/certs/<ca_cert_src basename>
  cert:        ditto
               default to /etc/pki/tls/certs/<cert_src basename>
  private_key: ditto
               default to /etc/pki/tls/private/<private_key_src basename>
```

#### logging_domain

The default DNS domain used to accept remote incoming logs from remote hosts. Default to "{{ ansible_domain if ansible_domain else ansible_hostname }}"

### Server performance optimization options

These variables are set in the same level of the `logging_inputs`, `logging_output`, and `logging_flows`.

- `logging_tcp_threads`: Input thread count listening on the plain tcp (ptcp) port. Default to `1`.
- `logging_udp_threads`: Input thread count listening on the udp port. Default to `1`.
- `logging_udp_system_time_requery`: Every `value` OS system calls, get the system time. Recommend not to set above 10. Default to `2` times.
- `logging_udp_batch_size`: Maximum number of udp messages per OS system call. Recommend not to set above 128. Default to `32`.
- `logging_server_queue_type`: Type of queue. `FixedArray` is available. Default to `LinkedList`.
- `logging_server_queue_size`: Maximum number of messages in the queue. Default to `50000`.
- `logging_server_threads`: Number of worker threads. Default to `logging_tcp_threads` + `logging_udp_threads`.

### Other options

These variables are set in the same level of the `logging_inputs`, `logging_output`, and `logging_flows`.

- `logging_enabled`: When 'true', logging role will deploy specified configuration file set. Default to 'true'.
- `logging_mark`: Mark message periodically by immark, if set to `true`. Default to `false`.
- `logging_mark_interval`: Interval for `logging_mark` in seconds. Default to `3600`.
- `logging_purge_confs`: `true` or `false`. If `logging_purge_confs` is set to
  `true`, remove files in rsyslog.d which do not belong to any rpm packages.
  That includes config files generated by the previous logging role run.   NOTE:
  If `/etc/rsyslog.conf` was modified, and you use `logging_purge_confs: true`,
  and you are not providing any `logging_inputs`, then the `rsyslog` package
  will be uninstalled and reinstalled in order to revert back to the original
  system default configuration.
- `logging_system_log_dir`: Directory where the local log output files are placed. Default to `/var/log`.
- `logging_manage_firewall`: If set to `true` and ports are found in the logging role
  parameters, configure the firewall for the ports using the firewall role.
  If set to `false`, the `logging role` does not manage the firewall.
  Default to `false`.
  NOTE: `logging_manage_firewall` is limited to *adding* ports.
  It cannot be used for *removing* ports.
  If you want to remove ports, you will need to use the firewall system
  role directly.
- `logging_manage_selinux`: If set to `true` and ports are found in the logging role
  parameters, configure the SELinux for the ports using the `selinux` role.
  If set to `false`, the `logging` role does not manage SELinux.
  Default to `false`.
  NOTE: `logging_manage_selinux` is limited to *adding* policy.
  It cannot be used for *removing* policy.
  If you want to remove policy, you will need to use the `selinux` role directly.
- `logging_certificates`: Information used to generate a private key and certificate.
  Default to `[]`.
  The value of `logging_certificates` is passed on to the `certificate_requests`
  in the `certificate` role and used to create the private key and certificate.
  For the supported parameters of `logging_certificates`,
  see the [`certificate_requests` role documentation section](https://github.com/linux-system-roles/certificate/#certificate_requests).
  Before running the `logging` role with `logging_certificates`, join the managed
  hosts to an IPA domain to make CA available to sign the certificate to be created.

  With this example, a private key `logging_cert.key` is generated in `/etc/pki/tls/private`
  and a certificate `logging_cert.crt` is in `/etc/pki/tls/certs` signed by
  the CA certificate managed by `ipa`.
```yaml
    logging_certificates:
      - name: logging_cert
        dns: ['localhost', 'www.example.com']
        ca: ipa
```
  The created private key and certificate are set with the ca certificate, e.g.,
  in `logging_pki_files` as follows:
```yaml
  logging_pki_files:
    - ca_cert: /etc/ipa/ca.crt
      cert: /etc/pki/tls/certs/logging_cert.crt
      private_key: /etc/pki/tls/private/logging_cert.key
```
  or in the `relp` parameters as follows:
```yaml
   logging_inputs:
     - name: relp_server
       type: relp
       tls: true
       ca_cert: /etc/ipa/ca.crt
       cert: /etc/pki/tls/certs/logging_cert.crt
       private_key: /etc/pki/tls/private/logging_cert.key
       [snip]
```
  NOTE: The `certificate` role, unless using IPA and joining the systems to an
  IPA domain, creates self-signed certificates, so you will need to explicitly
  configure trust, which is not currently supported by the system roles.

### Update and Delete

Due to the nature of ansible idempotency, if you run ansible-playbook multiple times without changing any variables and options, no changes are made from the second time. If some changes are made, only the rsyslog configuration files affected by the changes are recreated. To delete any existing rsyslog input or output config files generated by the previous ansible-playbook run, you need to add "state: absent" to the dictionary to be deleted (in this case, input_nameA and output_name0). And remove the flow dictionary related to the input and output as follows.

```yaml
logging_inputs:
  - name: input_nameA
    type: input_typeA
    state: absent
  - name: input_nameB
    type: input_typeB
logging_outputs:
  - name: output_name0
    type: output_type0
    state: absent
  - name: output_name1
    type: output_type1
logging_flows:
  - name: flow_nameY
    inputs: [input_nameB]
    outputs: [output_name1]
```

If you want to remove all the configuration files previously configured, in addition to setting `state: absent` to each logging_inputs and logging_outputs item, add `logging_enabled: false` to the configuration variables as follows. It will eliminate the global and common configuration files, as well.  Or, use `logging_purge_confs: true` to wipe out all previous configuration and replace it with your given configuration.

```yaml
logging_enabled: false
logging_inputs:
  - name: input_nameA
    type: input_typeA
    state: absent
  - name: input_nameB
    type: input_typeB
    state: absent
logging_outputs:
  - name: output_name0
    type: output_type0
    state: absent
  - name: output_name1
    type: output_type1
    state: absent
logging_flows:
  - name: flow_nameY
    inputs: [input_nameB]
    outputs: [output_name1]
```

## Configuration Examples

### Standalone configuration

Deploying `basics input` reading logs from systemd journal and implicit `files output` to write to the local files.

```yaml
---
- name: Deploying basics input and implicit files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: system_input
        type: basics
```

The following playbook generates the same logging configuration files.

```yaml
---
- name: Deploying basics input and files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: system_input
        type: basics
    logging_outputs:
      - name: files_output
        type: files
    logging_flows:
      - name: flow0
        inputs: [system_input]
        outputs: [files_output]
```

Deploying `basics input` reading logs from systemd unix socket and `files output` to write to the local files.

```yaml
---
- name: Deploying basics input using systemd unix socket and files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: system_input
        type: basics
        use_imuxsock: true
    logging_outputs:
      - name: files_output
        type: files
    logging_flows:
      - name: flow0
        inputs: [system_input]
        outputs: [files_output]
```

Deploying `basics input` reading logs from systemd journal and `files output` to write to the individually configured local files.

```yaml
---
- name: Deploying basic input and configured files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: system_input
        type: basics
    logging_outputs:
      - name: files_output0
        type: files
        severity: info
        exclude:
          - authpriv.none
          - auth.none
          - cron.none
          - mail.none
        path: /var/log/messages
      - name: files_output1
        type: files
        facility: authpriv,auth
        path: /var/log/secure
    logging_flows:
      - name: flow0
        inputs: [system_input]
        outputs: [files_output0, files_output1]
```

Deploying `files input` reading logs from local files and `files output` to write to the individually configured local files.

```yaml
---
- name: Deploying files input and configured files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: files_input0
        type: files
        input_log_path: /var/log/containerA/*.log
      - name: files_input1
        type: files
        input_log_path: /var/log/containerB/*.log
    logging_outputs:
      - name: files_output0
        type: files
        severity: info
        exclude:
          - authpriv.none
          - auth.none
          - cron.none
          - mail.none
        path: /var/log/messages
      - name: files_output1
        type: files
        facility: authpriv,auth
        path: /var/log/secure
    logging_flows:
      - name: flow0
        inputs: [files_input0, files_input1]
        outputs: [files_output0, files_output1]
```

Deploying `files input` reading logs from local files and `files output` to write to the local files based on the property-based filters.

```yaml
---
- name: Deploying files input and configured files output
  hosts: all
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: files_input0
        type: files
        input_log_path: /var/log/containerA/*.log
      - name: files_input1
        type: files
        input_log_path: /var/log/containerB/*.log
    logging_outputs:
      - name: files_output0
        type: files
        property: msg
        property_op: contains
        property_value: error
        path: /var/log/errors.log
      - name: files_output1
        type: files
        property: msg
        property_op: "!contains"
        property_value: error
        path: /var/log/others.log
    logging_flows:
      - name: flow0
        inputs: [files_input0, files_input1]
        outputs: [files_output0, files_output1]
```

### Client configuration

Deploying `basics input` reading logs from systemd journal and `forwards output` to forward the logs to the remote rsyslog.

```yaml
---
- name: Deploying basics input and forwards output
  hosts: clients
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: basic_input
        type: basics
    logging_outputs:
      - name: forward_output0
        type: forwards
        severity: info
        target: your_target_hostname
        udp_port: 514
      - name: forward_output1
        type: forwards
        facility: mail
        target: your_target_hostname
        tcp_port: 514
    logging_flows:
      - name: flows0
        inputs: [basic_input]
        outputs: [forward_output0, forward_output1]
```

Deploying `files input` reading logs from a local file and `forwards output` to forward the logs to the remote rsyslog over tls. Assuming the ca_cert, cert and key files are prepared at the specified paths on the control host. The files are deployed to the default location `/etc/pki/tls/certs/`, `/etc/pki/tls/certs/`, and `/etc/pki/tls/private`, respectively.

```yaml
---
- name: Deploying files input and forwards output with certs
  hosts: clients
  roles:
    - linux-system-roles.logging
  vars:
    logging_pki_files:
      - ca_cert_src: /local/path/to/ca_cert
        cert_src: /local/path/to/cert
        private_key_src: /local/path/to/key
    logging_inputs:
      - name: files_input
        type: files
        input_log_path: /var/log/containers/*.log
    logging_outputs:
      - name: forwards_output
        type: forwards
        target: your_target_host
        tcp_port: your_target_port
        pki_authmode: x509/name
        permitted_server: '*.example.com'
    logging_flows:
      - name: flows0
        inputs: [basic_input]
        outputs: [forwards-severity_and_facility]
```

### Server configuration

Deploying `remote input` reading logs from remote rsyslog and `remote_files output` to write the logs to the local files under the directory named by the remote host name.

```yaml
---
- name: Deploying remote input and remote_files output
  hosts: server
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: remote_udp_input
        type: remote
        udp_ports: [514, 1514]
      - name: remote_tcp_input
        type: remote
        tcp_ports: [514, 1514]
    logging_outputs:
      - name: remote_files_output
        type: remote_files
    logging_flows:
      - name: flow_0
        inputs: [remote_udp_input, remote_tcp_input]
        outputs: [remote_files_output]
```

Deploying `remote input` reading logs from remote rsyslog and `remote_files output` to write the logs to the configured local files with the tls setup supporting 20 clients. Assuming the ca_cert, cert and key files are prepared at the specified paths on the control host. The files are deployed to the default location `/etc/pki/tls/certs/`, `/etc/pki/tls/certs/`, and `/etc/pki/tls/private`, respectively.

```yaml
---
- name: Deploying remote input and remote_files output with certs
  hosts: server
  roles:
    - linux-system-roles.logging
  vars:
    logging_pki_files:
      - ca_cert_src: /local/path/to/ca_cert
        cert_src: /local/path/to/cert
        private_key_src: /local/path/to/key
    logging_inputs:
      - name: remote_tcp_input
        type: remote
        tcp_ports: [6514, 7514]
        permitted_clients: ['*.example.com', '*.test.com']
    logging_outputs:
      - name: remote_files_output0
        type: remote_files
        remote_log_path: /var/log/remote/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log
        async_writing: true
        client_count: 20
        io_buffer_size: 8192
      - name: remote_files_output1
        type: remote_files
        remote_sub_path: others/%FROMHOST%/%PROGRAMNAME:::secpath-replace%.log
    logging_flows:
      - name: flow_0
        inputs: [remote_udp_input, remote_tcp_input]
        outputs: [remote_files_output0, remote_files_output1]
```

### Client configuration with Relp

Deploying `basics input` reading logs from systemd journal and `relp output` to send the logs to the remote rsyslog over relp.

```yaml
---
- name: Deploying basics input and relp output
  hosts: clients
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: basic_input
        type: basics
    logging_outputs:
      - name: relp_client
        type: relp
        target: logging.server.com
        port: 20514
        tls: true
        ca_cert_src: /path/to/ca.pem
        cert_src: /path/to/client-cert.pem
        private_key_src: /path/to/client-key.pem
        pki_authmode: name
        permitted_servers:
          - '*.server.com'
    logging_flows:
      - name: flow
        inputs: [basic_input]
        outputs: [relp_client]
```

### Server configuration with Relp

Deploying `relp input` reading logs from remote rsyslog and `remote_files output` to write the logs to the local files under the directory named by the remote host name.

```yaml
---
- name: Deploying remote input and remote_files output
  hosts: server
  roles:
    - linux-system-roles.logging
  vars:
    logging_inputs:
      - name: relp_server
        type: relp
        port: 20514
        tls: true
        ca_cert_src: /path/to/ca.pem
        cert_src: /path/to/server-cert.pem
        private_key_src: /path/to/server-key.pem
        pki_authmode: name
        permitted_clients:
          - '*.client.com'
          - '*.example.com'
    logging_outputs:
      - name: remote_files_output
        type: remote_files
    logging_flows:
      - name: flow
        inputs: [relp_server]
        outputs: [remote_files_output]
```

### Port Managed by Firewall and SELinux Role

When a port is specified in the logging role configuration,
the firewall role is automatically included and the port
is managed by the firewalld.

The port is then configured by the `selinux` role and given
an appropriate syslog SELinux port type depending upon the
associated TLS value.

You can verify the changes by the following command-line.

For firewall,
```
firewall-cmd --list-port
```

For SELinux,
```
semanage port --list | grep "syslog"
```
The newly specified port will be added to this default set.
```
syslog_tls_port_t     tcp   6514, 10514
syslog_tls_port_t     udp   6514, 10514
syslogd_port_t        tcp   601, 20514
syslogd_port_t        udp   514, 601, 20514
```

## Providers

- Rsyslog

## Tests

tests/README.md - This documentation shows how to execute CI tests in the tests directory as well as how to debug when the test fails.

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##

## TODO: these constants must be documented
#'
#' The logging levels, names and values
#'
#' This list associates names to values and vice versa.\cr
#' Names and values are the same as in the python standard logging module.
#'
#' @export
#'
loglevels <- c(NOTSET = 0,
               FINEST = 1,
               FINER = 4,
               FINE = 7,
               DEBUG = 10,
               INFO = 20,
               WARNING = 30,
               WARN = 30,
               ERROR = 40,
               CRITICAL = 50,
               FATAL = 50)

namedLevel <- function(value)
  UseMethod("namedLevel")

namedLevel.default <- function(value) {
  loglevels[1]
}

namedLevel.character <- function(value) {
  position <- which(names(loglevels) == value)
  if (length(position) == 0) {
    position <- 1
  }
  loglevels[position][1]
}

namedLevel.numeric <- function(value) {
  position <- which(loglevels == value)
  if (length(position) == 0) {
    position <- 1
  }
  loglevels[position][1]
}

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##

#' @import methods
NULL

#' A tentative logging package.
#'
#' @description
#' A logging package emulating the Python logging package.
#'
#' @description
#' What you find here behaves similarly to what you also find in Python.
#' This includes hierarchic loggers, multiple handlers at each logger,
#' the possibility to specify a formatter for each handler (one default
#' formatter is given), same levels (names and numeric values) as Python's
#' logging package, a simple logging.BasicConfig function to quickly put
#' yourself in a usable situation...
#'
#' This package owes a lot to my employer, r-forge, the stackoverflow community,
#' Brian Lee Yung Rowe's futile package (v1.1) and the documentation of
#' the Python logging package.
#'
#' @details
#' Index:
#' \describe{
#' \item{\code{basicConfig}}{bootstrapping the logging package}
#' \item{\code{addHandler}}{add a handler to a logger}
#' \item{\code{getLogger}}{set defaults and get current values for a logger}
#' \item{\code{removeHandler}}{remove a handler from a logger}
#' \item{\code{setLevel}}{set logging.level for a logger}
#' }
#'
#' To use this package, include logging instructions in your code, possibly
#' like this:
#'
#' \code{library(logging)}\cr
#' \code{basicConfig()}\cr
#' \code{addHandler(writeToFile, logger="company", file="sample.log")}\cr
#' \code{loginfo("hello world", logger="")}\cr
#' \code{logwarn("hello company", logger="company.module")}
#'
#' The \code{basicConfig} function adds a console handler to the root logger,
#' while the explicitly called \code{addHandler} adds a file handler to the
#' 'company' logger. the effect of the above example is two lines on the
#' console and just one line in the \code{sample.log} file.
#'
#' The web pages for this package on r-forge are kept decently up to date
#' and contain a usable tutorial. Check the references.
#'
#' @references
#' the python logging module: \url{http://docs.python.org/library/logging.html}\cr
#' this package at github: \url{https://github.com/WLOGSolutions/r-logging/}
#'
#' @docType package
#' @name logging-package
#' @keywords package
NULL

## create the logging environment
logging.options <- new.env()

# Extracted from test.msg-composer.R:84

# setup ------------------------------------------------------------------------
library(testthat)
test_env <- simulate_test_env(package = "logging", path = "..")
attach(test_env, warn.conflicts = FALSE)

# prequel ----------------------------------------------------------------------
library(testthat)
library(logging)
context("Testing message composer functionality [test.msg-composer]")
test_setup <- function() {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    paste(record$levelname, record$logger, record$msg, sep = ":")
  }

  logReset()
  addHandler(mock_action,
             formatter = mock_formatter)

  return(test_env)
}

# test -------------------------------------------------------------------------
expect_no_error(loginfo(as.Date("2022-01-01")))

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##


#'
#' Predefined(sample) handler actions
#'
#' When you define a handler, you specify its name and the associated action.
#' A few predefined actions described below are provided.
#'
#' A handler action is a function that accepts a formated message and handler
#' configuration.
#'
#' Messages passed are filtered already regarding loglevel.
#'
#' \dots parameters are used by logging system to interact with the action. \dots can
#' contain \var{dry} key to inform action that it meant to initialize itself. In the case
#' action should return TRUE if initialization succeded.
#'
#' If it's not a dry run \dots contain the whole preformated \var{logging.record}.
#' A \var{logging.record} is a named list and has following structure:
#' \describe{
#'   \item{msg}{contains the real formatted message}
#'   \item{level}{message level as numeric}
#'   \item{levelname}{message level name}
#'   \item{logger}{name of the logger that generated it}
#'   \item{timestamp}{formatted message timestamp}
#' }
#'
#' @param msg A formated message to handle.
#' @param handler The handler environment containing its options. You can
#'   register the same action to handlers with different properties.
#' @param ... parameters provided by logger system to interact with the action.
#'
#' @examples
#' ## define your own function and register it with a handler.
#' ## author is planning a sentry client function.  please send
#' ## any interesting function you may have written!
#'
#' @name inbuilt-actions
NULL

#' @rdname inbuilt-actions
#'
#' @details
#' \code{writeToConsole} detects if crayon package is available and uses it
#' to color messages. The coloring can be switched off by means of configuring
#' the handler with \var{color_output} option set to FALSE.
#'
#' @export
#'
writeToConsole <- function(msg, handler, ...) {
  if (length(list(...)) && "dry" %in% names(list(...))) {
    if (!is.null(handler$color_output) && handler$color_output == FALSE) {
      handler$color_msg <- function(msg, level_name) msg
    } else {
      handler$color_msg <- .build_msg_coloring()
    }
    return(TRUE)
  }

  stopifnot(length(list(...)) > 0)

  level_name <- list(...)[[1]]$levelname
  msg <- handler$color_msg(msg, level_name)
  cat(paste0(msg, "\n"))
}

.build_msg_coloring <- function() {
  crayon_env <- tryCatch(asNamespace("crayon"),
                         error = function(e) NULL)

  default_color_msg <- function(msg, level_name) msg
  if (is.null(crayon_env)) {
    return(default_color_msg)
  }

  if (is.null(crayon_env$make_style) ||
      is.null(crayon_env$combine_styles) ||
      is.null(crayon_env$reset)) {
    return(default_color_msg)
  }

  color_msg <- function(msg, level_name) {
    style <- switch(level_name,
                    "FINEST" = crayon_env$make_style("gray80"),
                    "FINER" = crayon_env$make_style("gray60"),
                    "FINE" = crayon_env$make_style("gray60"),
                    "DEBUG" = crayon_env$make_style("deepskyblue4"),
                    "INFO" = crayon_env$reset,
                    "WARNING" = crayon_env$make_style("darkorange"),
                    "ERROR" = crayon_env$make_style("red4"),
                    "CRITICAL" =
                      crayon_env$combine_styles(crayon_env$bold,
                                                crayon_env$make_style("red1")),
                    crayon_env$make_style("gray100"))
    res <- paste0(style(msg), crayon_env$reset(""))
    return(res)
  }
  return(color_msg)
}


#' @rdname inbuilt-actions
#'
#' @details \code{writeToFile} action expects file path to write to under
#'  \var{file} key in handler options.
#'
#' @export
#'
writeToFile <- function(msg, handler, ...) {
  if (length(list(...)) && "dry" %in% names(list(...)))
    return(exists("file", envir = handler))
  cat(paste0(msg, "\n"), file = with(handler, file), append = TRUE)
}

## the single predefined formatter
defaultFormat <- function(record) {
  ## strip leading and trailing whitespace from the final message.
  msg <- trimws(record$msg)
  text <- paste(record$timestamp,
                paste(record$levelname, record$logger, msg, sep = ":"))
  return(text)
}

## default way of composing msg with parameters
defaultMsgCompose <- function(msg, ...) {
  optargs <- list(...)
  if (is.character(msg)) {
    ## invoked as ("printf format", arguments_for_format)
    if (length(optargs) > 0) {
      optargs <- lapply(optargs,
                        function(x) {
                          if (length(x) != 1)
                            x <- paste(x, collapse = ",")
                          x
                        })
    }

    # 8192 is limitation on fmt in sprintf
    if (any(nchar(msg) > 8192)) {
      if (length(optargs) > 0) {
        stop("'msg' length exceeds maximal format length 8192")
      }

      # else msg must not change in any way
      return(msg)
    }
    if (length(optargs) > 0) {
      msg <- do.call("sprintf", c(msg, optargs))
    }
    return(msg)
  }

  ## invoked as list of expressions
  ## this assumes that the function the user calls is two levels up, e.g.:
  ## loginfo -> .levellog -> logger$log -> .default_msg_composer
  ## levellog -> .levellog -> logger$log -> .default_msg_composer
  external_call <- sys.call(-3)
  external_fn <- eval(external_call[[1]])
  matched_call <- match.call(external_fn, external_call)
  matched_call <- matched_call[-1]
  matched_call_names <- names(matched_call)

  ## We are interested only in the msg and ... parameters,
  ## i.e. in msg and all parameters not explicitly declared
  ## with the function
  formal_names <- names(formals(external_fn))
  is_output_param <-
    matched_call_names == "msg" |
    !(matched_call_names %in% c(setdiff(formal_names, "...")))

  label <- lapply(matched_call[is_output_param], deparse)
  msg <- sprintf("%s: %s", label, c(msg, optargs))
  return(msg)
}

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##

#'
#' Entry points for logging actions
#'
#' Generate a log record and pass it to the logging system.\cr
#'
#' A log record gets timestamped and will be independently formatted by each
#' of the handlers handling it.\cr
#'
#' Leading and trailing whitespace is stripped from the final message.
#'
#' @param msg the textual message to be output, or the format for the \dots
#'  arguments
#' @param ... if present, msg is interpreted as a format and the \dots values
#'  are passed to it to form the actual message.
#' @param logger the name of the logger to which we pass the record
#'
#' @examples
#' logReset()
#' addHandler(writeToConsole)
#' loginfo('this goes to console')
#' logdebug('this stays silent')
#'
#' @name logging-entrypoints
#'
NULL

#' @rdname logging-entrypoints
#' @export
#'
logdebug <- function(msg, ..., logger = "") {
  .levellog(loglevels["DEBUG"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
logfinest <- function(msg, ..., logger = "") {
  .levellog(loglevels["FINEST"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
logfiner <- function(msg, ..., logger = "") {
  .levellog(loglevels["FINER"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
logfine <- function(msg, ..., logger = "") {
  .levellog(loglevels["FINE"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
loginfo <- function(msg, ..., logger = "") {
  .levellog(loglevels["INFO"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
logwarn <- function(msg, ..., logger = "") {
  .levellog(loglevels["WARN"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
#'
logerror <- function(msg, ..., logger = "") {
  .levellog(loglevels["ERROR"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#'
#' @param level The logging level
#' @export
levellog <- function(level, msg, ..., logger = "") {
  # just calling .levellog
  # do not simplify it as call sequence sould be same
  #   as for other logX functions
  .levellog(level, msg, ..., logger = logger)
}

.levellog <- function(level, msg, ..., logger = "") {
  if (is.character(logger)) {
    logger <- getLogger(logger)
  }
  logger$log(level, msg, ...)
}



#'
#' Set defaults and get the named logger.
#'
#' Make sure a logger with a specific name exists and return it as a
#' \var{Logger} S4 object.  if not yet present, the logger will be created and
#' given the values specified in the \dots arguments.
#'
#' @param name The name of the logger
#' @param ... Any properties you may want to set in the newly created
#'    logger. These have no effect if the logger is already present.
#'
#' @return The logger retrieved or registered.
#'
#' @examples
#' getLogger()  # returns the root logger
#' getLogger('test.sub')  # constructs a new logger and returns it
#' getLogger('test.sub')  # returns it again
#'
#' @export
#'
getLogger <- function(name = "", ...) {
  if (name == "") {
    fullname <- "logging.ROOT"
  } else {
    fullname <- paste("logging.ROOT", name, sep = ".")
  }

  if (!exists(fullname, envir = logging.options)) {
    logger <- Logger$new(name = name,
                         handlers = list(),
                         level = namedLevel("NOTSET"))
    updateOptions.environment(logger, ...)
    logging.options[[fullname]] <- logger

    if (fullname == "logging.ROOT") {
      .basic_config(logger)
    }
  }
  logging.options[[fullname]]
}



#'
#' Bootstrapping the logging package.
#'
#' \code{basicConfig} and \code{logReset} provide a way to put the logging package
#' in a know initial state.
#'
#' @examples
#' basicConfig()
#' logdebug("not shown, basic is INFO")
#' logwarn("shown and timestamped")
#' logReset()
#' logwarn("not shown, as no handlers are present after a reset")
#'
#' @name bootstrapping
NULL

#' @rdname bootstrapping
#'
#' @details
#' \code{basicConfig} creates the root logger, attaches a console handler(by
#' \var{basic.stdout} name) to it and sets the level of the handler to
#' \code{level}. You must not call \code{basicConfig} to for logger to work any more:
#' then root logger is created it gets initialized by default the same way as
#' \code{basicConfig} does. If you need clear logger to fill with you own handlers
#' use \code{logReset} to remove all default handlers.
#'
#' @param level The logging level of the root logger. Defaults to INFO. Please do notice that
#'   this has no effect on the handling level of the handler that basicConfig attaches to the
#'   root logger.
#'
#' @export
#'
basicConfig <- function(level = 20) {
  root_logger <- getLogger()

  updateOptions(root_logger, level = namedLevel(level))
  .basic_config(root_logger)

  invisible()
}

#' Called from basicConfig and while creating rootLogger.
#' @noRd
.basic_config <- function(root_logger) {
  stopifnot(root_logger$name == "")
  root_logger$addHandler("basic.stdout", writeToConsole)
}


#' @rdname bootstrapping
#'
#' @details
#' \code{logReset} reinitializes the whole logging system as if the package had just been
#' loaded except it also removes all default handlers. Typically, you would want to call
#' \code{basicConfig} immediately after a call to \code{logReset}.
#'
#' @export
#'
logReset <- function() {
  ## reinizialize the whole logging system

  ## remove all content from the logging environment
  rm(list = ls(logging.options), envir = logging.options)

  root_logger <- getLogger(level = "INFO")
  root_logger$removeHandler("basic.stdout")

  invisible()
}



#'
#' Add a handler to or remove one from a logger.
#'
#' Use this function to maintain the list of handlers attached to a logger.\cr
#' \cr
#' \code{addHandler} and \code{removeHandler} are also offered as methods of the
#' \var{Logger} S4 class.
#'
#' @details
#' Handlers are implemented as environments. Within a logger a handler is
#' identified by its \var{name} and all handlers define at least the
#' three variables:
#' \describe{
#'   \item{level}{all records at level lower than this are skipped.}
#'   \item{formatter}{a function getting a record and returning a string}
#'   \item{\code{action(msg, handler)}}{a function accepting two parameters: a
#'      formatted log record and the handler itself. making the handler a
#'      parameter of the action allows us to have reusable action functions.}
#' }
#'
#' Being an environment, a handler may define as many variables as you
#' think you need.  keep in mind the handler is passed to the action
#' function, which can check for existence and can use all variables that
#' the handler defines.
#'
#' @param handler The name of the handler, or its action
#' @param logger the name of the logger to which to attach the new handler,
#'   defaults to the root logger.
#'
#' @examples
#' logReset()
#' addHandler(writeToConsole)
#' names(getLogger()[["handlers"]])
#' loginfo("test")
#' removeHandler("writeToConsole")
#' logwarn("test")
#'
#' @name handlers-management
NULL

#' @rdname handlers-management
#'
#' @param ... Extra parameters, to be stored in the handler list
#'
#' \dots may contain extra parameters that will be passed to the handler
#' action. Some elements in the \dots will be interpreted here.
#'
#' @export
#'
addHandler <- function(handler, ..., logger = "") {
  if (is.character(logger)) {
    logger <- getLogger(logger)
  }

  ## this part has to be repeated here otherwise the called function
  ## will deparse the argument to 'handler', the formal name given
  ## here to the parameter
  if (is.character(handler)) {
    logger$addHandler(handler, ...)
  } else {
    handler_name <- deparse(substitute(handler))
    logger$addHandler(handler = handler_name, action = handler, ...)
  }
}

#' @rdname handlers-management
#' @export
#'
removeHandler <- function(handler, logger = "") {
  if (is.character(logger)) {
    logger <- getLogger(logger)
  }
  if (!is.character(handler)) {
    # handler was passed as its action
    handler <- deparse(substitute(handler))
  }
  logger$removeHandler(handler)
}

#'
#' Retrieves a handler from a logger.
#'
#' @description
#' Handlers are not uniquely identified by their name. Only within the logger to which
#' they are attached is their name unique. This function is here to allow you grab a
#' handler from a logger so you can examine and alter it.
#'
#' @description
#' Typical use of this function is in \code{setLevel(newLevel, getHandler(...))}.
#'
#' @param handler The name of the handler, or its action.
#' @param logger Optional: the name of the logger. Defaults to the root logger.
#'
#' @return The retrieved handler object. It returns NULL if handler is not registerd.
#'
#' @examples
#' logReset()
#' addHandler(writeToConsole)
#' getHandler("basic.stdout")
#'
#' @export
#'
getHandler <- function(handler, logger = "") {
  if (is.character(logger)) {
    logger <- getLogger(logger)
  }
  if (!is.character(handler)) {
    # handler was passed as its action
    handler <- deparse(substitute(handler))
  }
  logger$getHandler(handler)
}



#'
#' Set \var{logging.level} for the object.
#'
#' Alter an existing logger or handler, setting its \var{logging.level} to a new
#' value. You can access loggers by name, while you must use \code{getHandler} to
#' get a handler.
#'
#' @param level The new level for this object. Can be numeric or character.
#' @param container a logger, its name or a handler. Default is root logger.
#'
#' @examples
#' basicConfig()
#' setLevel("FINEST")
#' setLevel("DEBUG", getHandler("basic.stdout"))
#'
#' @export
#'
setLevel <- function(level, container = "") {
  if (is.null(container)) {
    stop("NULL container provided: cannot set level for NULL container")
  }

  if (is.character(container)) {
    container <- getLogger(container)
  }
  assign("level", namedLevel(level), container)
}

#'
#' Sets message composer for logger.
#'
#' Message composer is used to compose log message out of formating string and arguments.
#' It is function with signature \code{function(msg, ...)}. Formating message is passed under msg
#' and formating arguments are passed as \code{...}.
#'
#' If message composer is not set default is in use (realized with \code{sprintf}). If message
#' composer is not set for sub-logger, parent's message composer will be used.
#'
#' @param composer_f message composer function (type: function(msg, ...))
#' @param container name of logger to reser message composer for (type: character)
#'
#' @examples
#' setMsgComposer(function(msg, ...) paste0("s-", msg, "-e"))
#' loginfo("a message") # will log '<TS> INFO::s-a message-e'
#' resetMsgComposer()
#' loginfo("a message") # will log '<TS> INFO::a message'
#'
#' @export
#'
setMsgComposer <- function(composer_f, container = "") {
  if (is.null(container)) {
    stop("NULL container provided: cannot set message composer for NULL container")
  }

  if (is.character(container)) {
    container <- getLogger(container)
  }
  container$setMsgComposer(composer_f)
  assign("msg_composer", composer_f, container)
}

#'
#' Resets previously set message composer.
#'
#' @param container name of logger to reser message composer for (type: character)
#'
#' @export
#'
resetMsgComposer <- function(container = "") {
  if (is.null(container)) {
    stop("NULL container provided: cannot resset message composer for NULL container")
  }

  if (is.character(container)) {
    container <- getLogger(container)
  }
  assign("msg_composer", function() NULL, container)
}

library(testthat)
library(logging)

context("Testing data interactions [test.data.interaction]")

test_that("000/getLoggerWithoutInitializingDoesNotCrash", {
  root_logger <- getLogger("")

  succeed()
})

test_that("001/defaultLoggingLevelIsINFO", {
  basicConfig()

  root_logger <- getLogger("")
  expect <- logging:::loglevels["INFO"]

  expect_equal(root_logger[["level"]], expect)
})

test_that("002/canInitializeTwice", {
  basicConfig()

  basicConfig()
  root_logger <- getLogger("")
  expect <- logging:::loglevels["INFO"]

  expect_equal(root_logger[["level"]], expect)
})

test_that("003/sameNameMeansSameObject", {
  basicConfig()

  root1 <- getLogger("abc")
  root2 <- getLogger("abc")

  expect_identical(root1, root2)
})

test_that("004/noNameMeansRoot", {
  root_logger1 <- getLogger("")
  root_logger2 <- getLogger()

  expect_identical(root_logger1, root_logger2)
})

test_that("500.basicConfigSetsLevelOfHandler", {
  logReset()
  basicConfig("DEBUG")

  root_logger <- getLogger("")
  expect <- logging:::loglevels["NOTSET"]
  current <- root_logger$getHandler("basic.stdout")[["level"]]

  expect_equal(current, expect)

  logReset()
  basicConfig("ERROR")
  root_logger <- getLogger("")  ## needed: `logReset` unlinked the old one
  expect <- logging:::loglevels["NOTSET"]
  current <- root_logger$getHandler("basic.stdout")[["level"]]

  expect_equal(current, expect)
})


test_that("canFindLoggingLevels", {
  expect_equal(logging:::loglevels[["NOTSET"]], 0)
  expect_equal(logging:::loglevels[["DEBUG"]], 10)
  expect_equal(logging:::loglevels[["INFO"]], 20)
  expect_equal(logging:::loglevels[["WARN"]], 30)
  expect_equal(logging:::loglevels[["ERROR"]], 40)
  expect_equal(logging:::loglevels[["FATAL"]], 50)
})

test_that("fineLevelsAreOrdered", {
  expect_true(logging:::loglevels[["FINEST"]] < logging:::loglevels[["FINER"]])
  expect_true(logging:::loglevels[["FINER"]] < logging:::loglevels[["FINE"]])
  expect_true(logging:::loglevels[["FINE"]] < logging:::loglevels[["DEBUG"]])
})

test_that("canSetLoggerLevelByNamedValue", {
  logReset()
  basicConfig()

  setLevel(logging:::loglevels["DEBUG"], "")

  root_logger <- getLogger("")
  expect <- logging:::loglevels["DEBUG"]
  expect_equal(root_logger[["level"]], expect)
})

test_that("canSetLoggerLevelByName", {
  logReset()
  basicConfig()

  setLevel("DEBUG", "")

  root_logger <- getLogger("")
  expect <- logging:::loglevels["DEBUG"]
  expect_equal(root_logger[["level"]], expect)
})



test_setup <- function(...) {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    msg <- trimws(record$msg)
    paste(record$levelname, record$logger, msg, sep = ":")
  }

  logReset()
  addHandler(mock_action, formatter = mock_formatter, ...)

  return(test_env)
}


test_that("recordIsEmitted/rootToRoot", {
  env <- test_setup()

  logfinest("test")
  logfiner("test")
  logfine("test")
  logdebug("test")
  loginfo("test")
  logerror("test")

  expect_equal(length(env$logged), 2)
})

test_that("recordIsEmitted/tooDeep", {
  env <- test_setup(logger = "too.deep")

  logfinest("test")
  logfiner("test")
  logfine("test")
  logdebug("test")
  loginfo("test")
  logerror("test")

  expect_equal(length(env$logged), 0)
})

test_that("recordIsEmitted/unrelated", {
  env <- test_setup(logger = "too.deep")

  logdebug("test", logger = "other.branch")
  loginfo("test", logger = "other.branch")
  logerror("test", logger = "other.branch")

  expect_equal(length(env$logged), 0)
})

test_that("recordIsEmitted/deepToRoot", {
  env <- test_setup(logger = "")

  logdebug("test", logger = "other.branch")
  loginfo("test", logger = "other.branch")
  logerror("test", logger = "other.branch")

  expect_equal(length(env$logged), 2)
})

test_that("recordIsEmitted/deepToRoot.DI.dropped", {
  env <- test_setup(level = "DEBUG", logger = "")

  setLevel("INFO", "other.branch")
  logdebug("test", logger = "other.branch")
  loginfo("test", logger = "other.branch")
  logerror("test", logger = "other.branch")

  expect_equal(length(env$logged), 2)
})

test_that("recordIsEmitted/deepToRoot.DD.passed", {
  env <- test_setup(level = "DEBUG", logger = "")

  setLevel("DEBUG", "other.branch")
  setLevel("DEBUG", "")
  logdebug("test", logger = "other.branch")
  loginfo("test", logger = "other.branch")
  logerror("test", logger = "other.branch")

  expect_equal(length(env$logged), 3)
})


test_that("formattingRecord/lengthZero", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("test '%s'", numeric(0))

  expect_equal(env$logged, "INFO::test ''")
})

test_that("formattingRecord/lengthOne", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("test '%s'", 12)

  expect_equal(env$logged, "INFO::test '12'")
})

test_that("formattingRecord/lengthMore", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("test '%s'", c(0, 1, 2))

  expect_equal(env$logged, "INFO::test '0,1,2'")
})

test_that("formattingRecord/moreArguments", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("%s: %d", "name", 123)

  expect_equal(env$logged, "INFO::name: 123")


  env$logged <- NULL

  loginfo("%s: %0.2f", "name", 123.0)

  expect_equal(env$logged, "INFO::name: 123.00")
})

test_that("formattingRecord/moreArguments.lengthMore", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("%s '%s'", "name", c(0, 1, 2))

  expect_equal(env$logged, "INFO::name '0,1,2'")
})

test_that("formattingRecord/strips.whitespace", {
  env <- test_setup(level = "DEBUG", logger = "")

  loginfo("a string with trailing whitespace \n")

  expect_equal(env$logged, "INFO::a string with trailing whitespace")


  env$logged <- NULL

  loginfo("  this string has also leading whitespace   ")

  expect_equal(env$logged, "INFO::this string has also leading whitespace")
})

library(testthat)
library(logging)

context("Testing message composer functionality [test.msg-composer]")

test_setup <- function() {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    paste(record$levelname, record$logger, record$msg, sep = ":")
  }

  logReset()
  addHandler(mock_action,
             formatter = mock_formatter)

  return(test_env)
}

## test functions

test_that("msgcomposer/basic_funcionality", {
  env <- test_setup()

  setMsgComposer(function(msg, ...) { paste(msg, "comp") })
  loginfo("test")

  expect_equal(env$logged, "INFO::test comp")
})

test_that("msgcomposer/composer_signature_validation", {
  env <- test_setup()

  expect_error({
    setMsgComposer(function(msgX, ...) { paste(msg, "comp") })
  })
})

test_that("msgcomposer/reset_composer", {
  env <- test_setup()

  setMsgComposer(function(msg, ...) { paste(msg, "comp") })
  resetMsgComposer()
  loginfo("test")

  expect_equal(env$logged, "INFO::test")
})

test_that("msgcomposer/set_sublogger_composer", {
  env <- test_setup()

  setMsgComposer(function(msg, ...) { paste(msg, "comp") }, container = "named")
  loginfo("test")
  loginfo("test", logger = "named")

  expect_equal(env$logged, c("INFO::test", "INFO:named:test comp"))
})

test_that("msgcomposer/set_sublogger_composer_directly", {
  env <- test_setup()

  getLogger("named")$setMsgComposer(function(msg, ...) { paste(msg, "comp") })
  loginfo("test")
  loginfo("test", logger = "named")

  expect_equal(env$logged, c("INFO::test", "INFO:named:test comp"))
})

test_that("msgcomposer/composer_signature_validation_directly", {
  env <- test_setup()

  expect_error({
    getLogger("named")$setMsgComposer(function(msgX, ...) { paste(msg, "comp") })
  })
})

library(testthat)
library(logging)

context("Testing loglevel inheritance [test.level-inheritance]")

test_setup <- function() {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    paste(record$levelname, record$logger, record$msg, sep = ":")
  }

  logReset()
  addHandler(mock_action,
             formatter = mock_formatter)

  test_env$mock_action <- mock_action
  return(test_env)
}

test_that("leveliherit/sublogger_default_to_inherit_parent_level", {
  env <- test_setup()

  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, "INFO:named:test")
  env$logged <- c()

  setLevel("DEBUG")
  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO:named:test", "DEBUG:named:test"))
})

test_that("leveliherit/sublogger_enforced_level_used", {
  env <- test_setup()

  setLevel("DEBUG")
  setLevel("INFO", container = "named")
  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO:named:test"))
})

test_that("leveliherit/handler_default_to_inherit_parent_level", {
  env <- test_setup()

  mock_action2 <- env$mock_action
  addHandler(mock_action2, formatter = function(record) {
    paste(record$levelname, record$logger, record$msg, sep = "|")
  }, logger = "named")

  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO|named|test", "INFO:named:test"))
  env$logged <- c()

  setLevel("DEBUG")
  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO|named|test", "INFO:named:test",
                             "DEBUG|named|test", "DEBUG:named:test"))
})

test_that("leveliherit/handler_inherit_enforced_sublogger_level", {
  env <- test_setup()

  setLevel("DEBUG")
  setLevel("INFO", container = "named")

  mock_action2 <- env$mock_action
  addHandler(mock_action2, formatter = function(record) {
    paste(record$levelname, record$logger, record$msg, sep = "|")
  }, logger = "named")

  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO|named|test", "INFO:named:test"))
  env$logged <- c()

  setLevel("INFO")
  setLevel("DEBUG", container = "named")

  logdebug("test", logger = "named")

  expect_equal(env$logged, c("DEBUG|named|test"))
})

test_that("leveliherit/handler_enforced_level_used", {
  env <- test_setup()

  setLevel("DEBUG")

  mock_action2 <- env$mock_action
  addHandler(mock_action2, formatter = function(record) {
    paste(record$levelname, record$logger, record$msg, sep = "|")
  }, logger = "named", level = "INFO")

  loginfo("test", logger = "named")
  logdebug("test", logger = "named")

  expect_equal(env$logged, c("INFO|named|test", "INFO:named:test", "DEBUG:named:test"))
})

library(testthat)
library(logging)

context("Testing formatting of mess and exprs [test.automatic-format]")

test_setup <- function() {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    paste(record$levelname, record$logger, record$msg, sep = ":")
  }

  logReset()
  addHandler(mock_action,
             level = "DEBUG",
             formatter = mock_formatter)

  return(test_env)
}

## test functions

test_that("autoformat/one_numeric_literal", {
  env <- test_setup()

  loginfo(12)

  expect_equal(env$logged, "INFO::12: 12")
})

test_that("autoformat/more_numeric_literals", {
  env <- test_setup()

  loginfo(12, 1 + 1, 2 * 2)

  expect_equal(env$logged,
               c("INFO::12: 12", "INFO::1 + 1: 2", "INFO::2 * 2: 4"))
})

test_that("autoformat/one_numeric_variable", {
  env <- test_setup()

  a <- 1
  loginfo(a)

  expect_equal(env$logged, "INFO::a: 1")
})

test_that("autoformat/one_numeric_expression", {
  env <- test_setup()

  loginfo(3 + 5)

  expect_equal(env$logged, "INFO::3 + 5: 8")
})

test_that("autoformat/explicit_msg_parameter", {
  env <- test_setup()

  loginfo(logger = getLogger(), msg = 3 + 5)

  expect_equal(env$logged, "INFO::3 + 5: 8")
})

test_that("autoformat/shifted_msg_parameter", {
  env <- test_setup()

  loginfo(logger = getLogger(), 3 + 5)

  expect_equal(env$logged, "INFO::3 + 5: 8")
})

test_that("autoformat/levellog", {
  env <- test_setup()

  levellog("INFO", 3 + 5)

  expect_equal(env$logged, "INFO::3 + 5: 8")
})

library(testthat)
library(logging)

context("Testing handlers management [test.handlers]")

test_that("looksForHandlersInRootLogger", {
  logReset()
  basicConfig()

  expect_identical(getHandler("basic.stdout"),
                   getLogger()[["handlers"]][[1]])
})

test_that("lookingForHandlersInObject", {
  logReset()
  basicConfig()

  expect_identical(getLogger()$getHandler("basic.stdout"),
                   getLogger()[["handlers"]][[1]])
})

test_that("addingANewHandler", {
  logReset()
  basicConfig()

  addHandler(writeToConsole)

  expect_equal(length(with(getLogger(), names(handlers))), 2)
  expect_true("writeToConsole" %in% with(getLogger(), names(handlers)))
})

test_that("gettingHandler", {
  logReset()
  basicConfig()

  addHandler(writeToConsole)

  expect_true(!is.null(getHandler("writeToConsole")))
  expect_true(!is.null(getHandler(writeToConsole)))
})

test_that("gettingHandler/oo", {
  logReset()
  basicConfig()

  log <- getLogger("testLogger")
  log$addHandler(writeToConsole)

  expect_true(!is.null(log$getHandler("writeToConsole")))
  expect_true(!is.null(log$getHandler(writeToConsole)))
})

test_that("removingOneHandlerByName", {
  logReset()
  basicConfig()

  addHandler(writeToConsole)
  removeHandler("writeToConsole")

  expect_equal(length(with(getLogger(), names(handlers))), 1)
  expect_false("writeToConsole" %in% with(getLogger(), names(handlers)))
})

test_that("removingOneHandlerByAction", {
  logReset()
  basicConfig()

  addHandler(writeToConsole)

  expect_equal(length(with(getLogger(), names(handlers))), 2)

  removeHandler(writeToConsole)

  expect_equal(length(with(getLogger(), names(handlers))), 1)
  expect_false("writeToConsole" %in% with(getLogger(), names(handlers)))
})

test_that("removingOneHandlerByAction/oo", {
  logReset()
  basicConfig()

  log <- getLogger("testLogger")
  log$addHandler(writeToConsole)

  expect_equal(length(with(log, names(handlers))), 1)

  log$removeHandler(writeToConsole)

  expect_equal(length(with(log, names(handlers))), 0)
  expect_false("writeToConsole" %in% with(log, names(handlers)))
})

test_that("setLevel verifies if container is not NULL", {
  logReset()
  basicConfig()

  expect_error(setLevel("INFO", NULL),
               "NULL container provided: cannot set level for NULL container",
               fixed = TRUE)
})

test_that("addHandler verifies if action provided", {
  logReset()
  basicConfig()

  expect_error(addHandler("handlerName"),
               "No action for the handler provided",
               fixed = TRUE)
})

test_that("setLevel verifies level value", {
  logReset()
  setLevel(150) # invalid level

  expect_equal(getLogger()$getLevel(), loglevels["NOTSET"])

  logReset()
  setLevel(TRUE) # invalid level

  expect_equal(getLogger()$getLevel(), loglevels["NOTSET"])

  logReset()
  setLevel("INVALID") # invalid level

  expect_equal(getLogger()$getLevel(), loglevels["NOTSET"])
})


test_that("setLevel verifies level value/oo", {
  logReset()
  log <- getLogger("testLogger")

  log$setLevel(150) # invalid level

  expect_equal(log$getLevel(), loglevels["NOTSET"])


  log$setLevel(TRUE) # invalid level

  expect_equal(log$getLevel(), loglevels["NOTSET"])


  log$setLevel("INVALID") # invalid level

  expect_equal(log$getLevel(), loglevels["NOTSET"])
})


test_that("setting level with updateOptions", {
  logReset()
  basicConfig()

  updateOptions("", level = "WARN")

  expect_equal(getLogger()$getLevel(), loglevels["WARN"])
})


test_file_name <- file.path(tempdir(), c("1", "2", "3"))
teardown({
  unlink(test_file_name, force = TRUE)
})

test_that("loggingToConsole", {
  logReset()
  basicConfig()

  logdebug("test %d", 2)
  loginfo("test %d", 2)

  succeed()
})

test_that("loggingToFile", {
  logReset()
  unlink(test_file_name, force = TRUE)

  getLogger()$setLevel("FINEST")
  addHandler(writeToFile, file = test_file_name[[1]], level = "DEBUG")

  expect_equal(with(getLogger(), names(handlers)), c("writeToFile"))

  logerror("test %d", 1)
  logwarn("test %d", 3)
  loginfo("test %d", 1)
  logdebug("test %d", 2)
  logfinest("test %d", 4)
  logfiner("test %d", 4)
  logfine("test %d", 4)

  succeed()
})

test_that("loggingToFile.oo", {
  logReset()
  unlink(test_file_name, force = TRUE)

  log <- getLogger()
  log$setLevel("FINEST")
  log$addHandler(writeToFile, file = test_file_name[[1]], level = "DEBUG")

  expect_equal(with(log, names(handlers)), c("writeToFile"))

  log$error("test %d", 1)
  log$warn("test %d", 3)
  log$info("test %d", 1)
  log$debug("test %d", 2)
  log$finest("test %d", 4)
  log$finer("test %d", 4)
  log$fine("test %d", 4)

  succeed()
})

library(testthat)
library(logging)

context("Testing formatting limitation handling [test.format-limitations]")

test_setup <- function() {
  test_env <- new.env(parent = emptyenv())
  test_env$logged <- NULL

  mock_action <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
      return(TRUE)
    test_env$logged <- c(test_env$logged, msg)
  }
  mock_formatter <- function(record) {
    paste(record$levelname, record$logger, record$msg, sep = ":")
  }

  logReset()
  addHandler(mock_action,
             level = "DEBUG",
             formatter = mock_formatter)

  return(test_env)
}

## test functions

test_that("formatlimits/up_to_limit_w_formating", {
  env <- test_setup()

  msg0_9 <- paste(0:9, collapse = "")
  msg8190 <- paste(rep(msg0_9, 819), collapse = "")
  stopifnot(nchar(msg8190) == 8190)

  msg8190_s <- paste0(msg8190, "%s")
  stopifnot(nchar(msg8190_s) == 8192)

  loginfo(msg8190_s, paste0(".", msg0_9))

  expect_equal(env$logged, paste0("INFO::", msg8190, ".", msg0_9))
})

test_that("formatlimits/over_limit_w_formatting_fails", {
  env <- test_setup()

  msg0_9 <- paste(0:9, collapse = "")
  msg8190 <- paste(rep(msg0_9, 819), collapse = "")
  stopifnot(nchar(msg8190) == 8190)

  msg8190_sX <- paste0(msg8190, "%s", "X")
  stopifnot(nchar(msg8190_sX) == 8193)

  expect_error({
    loginfo(msg8190_sX, paste0(".", msg0_9))
  },
  regexp = "'msg' length exceeds maximal format length 8192")
})

test_that("formatlimits/over_limit_wo_format_args_passes", {
  env <- test_setup()

  msg0_9 <- paste(0:9, collapse = "")
  msg8190 <- paste(rep(msg0_9, 819), collapse = "")
  stopifnot(nchar(msg8190) == 8190)

  msg8190_sX <- paste0(msg8190, "%s", "X")
  stopifnot(nchar(msg8190_sX) == 8193)

  loginfo(msg8190_sX)

  expect_equal(env$logged, paste0("INFO::", msg8190_sX))
})

test_that("formatlimits/over_limit_wo_formatting", {
  env <- test_setup()

  msg0_9 <- paste(0:9, collapse = "")
  msg8190 <- paste(rep(msg0_9, 819), collapse = "")
  stopifnot(nchar(msg8190) == 8190)

  # long message without formatting
  msg8190_x2 <- paste0(msg8190, ".", msg8190)
  stopifnot(nchar(msg8190_x2) == 2 * 8190 + 1)

  loginfo(msg8190_x2)

  expect_equal(env$logged, paste0("INFO::", msg8190_x2))
  env$logged <- c()

  # long message with %% at end
  msg8190_m_perc <- paste0(msg8190, "%%", "X")
  stopifnot(nchar(msg8190_m_perc) == 8193)

  loginfo(msg8190_m_perc)

  expect_equal(env$logged, paste0("INFO::", msg8190_m_perc))
  env$logged <- c()

  # long message with %% at start
  msg8190_perc_m <- paste0("%%", "X", msg8190)
  stopifnot(nchar(msg8190_perc_m) == 8193)

  loginfo(msg8190_perc_m)

  expect_equal(env$logged, paste0("INFO::", msg8190_perc_m))
})

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##

Logger <- setRefClass(
  "Logger",
  fields = list(
    name = "character",
    handlers = "list",
    level = "numeric",
    msg_composer = "function"),
  methods = list(
    getParent = function() {
      # split the name on the '.'
      parts <- unlist(strsplit(name, ".", fixed = TRUE))
      removed <- parts[-length(parts)] # except the last item
      parent_name <- paste(removed, collapse = ".")
      return(getLogger(parent_name))
    },

    getMsgComposer = function() {
      if (!is.null(msg_composer) && !is.null(functionBody(msg_composer))) {
        return(msg_composer)
      }
      if (name != "") {
        parent_logger <- getParent()
        return(parent_logger$getMsgComposer())
      }
      return(defaultMsgCompose)
    },

    setMsgComposer = function(composer_f) {
      if (!is.function(composer_f)
          || paste(formalArgs(composer_f), collapse = ", ") != "msg, ...") {
        stop(paste("message composer(passed as composer_f) must be function",
                   " with signature function(msg, ...)"))
      }
      msg_composer <<- composer_f
    },

    .deducelevel = function(initial_level = loglevels[["NOTSET"]]) {
      if (initial_level != loglevels[["NOTSET"]]) {
        # it's proper level (set: Not for inheritance)
        return(initial_level)
      }

      if (level != loglevels[["NOTSET"]]) {
        return(level)
      }

      if (name == "") {
        # assume it's FINEST, as root logger cannot inherit
        return(loglevels[["FINEST"]])
      }

      # ask parent for level
      parent_logger <- getParent()
      return(parent_logger$.deducelevel())
    },

    .logrecord = function(record) {
      logger_level <- .deducelevel(level)
      if (record$level >= logger_level) {
        for (handler in handlers) {
          handler_level <- .deducelevel(with(handler, level))
          if (record$level >= handler_level) {
            action <- with(handler, action)
            formatter <- with(handler, formatter)
            action(formatter(record), handler, record)
          }
        }
      }

      if (name != "") {
        parent_logger <- getParent()
        parent_logger$.logrecord(record)
      }
      invisible(TRUE)
    },

    log = function(msglevel, msg, ...) {
      msglevel <- namedLevel(msglevel)
      if (msglevel < level) {
        return(invisible(FALSE))
      }
      ## fine, we create the record and pass it to all handlers attached to the
      ## loggers from here up to the root.
      record <- list()

      composer_f <- getMsgComposer()
      record$msg <- composer_f(msg, ...)
      record$timestamp <- sprintf("%s", Sys.time())
      record$logger <- name
      record$level <- msglevel
      record$levelname <- names(which(loglevels == record$level)[1])

      ## cascade action in private method.
      .logrecord(record)
    },

    setLevel = function(new_level) {
      new_level <- namedLevel(new_level)
      level <<- new_level
    },

    getLevel = function() level,

    getHandler = function(handler) {
      if (!is.character(handler))
        handler <- deparse(substitute(handler))
      handlers[[handler]]
    },

    removeHandler = function(handler) {
      if (!is.character(handler))  # handler was passed as its action
        handler <- deparse(substitute(handler))
      handlers <<- handlers[!(names(handlers) == handler)]
    },

    addHandler = function(handler, ..., level = 0, formatter = defaultFormat) {
      handler_env <- new.env()
      if (is.character(handler)) {
        ## first parameter is handler name
        handler_name <- handler
        ## and hopefully action is in the dots
        params <- list(...)
        if ("action" %in% names(params)) {
          the_action <- params[["action"]]
        } else if (length(params) > 0 && is.null(names(params)[[1]])) {
          the_action <- params[[1]]
        } else {
          stop("No action for the handler provided")
        }

        assign("action", the_action, handler_env)
      } else {
        ## first parameter is handler action, from which we extract the name
        updateOptions.environment(handler_env, action = handler)
        handler_name <- deparse(substitute(handler))
      }
      updateOptions.environment(handler_env, ...)
      assign("level", namedLevel(level), handler_env)
      assign("formatter", formatter, handler_env)
      removeHandler(handler_name)

      if (with(handler_env, action)(NA, handler_env, dry = TRUE) == TRUE) {
        handlers[[handler_name]] <<- handler_env
      }
    },

    finest = function(...) log(loglevels["FINEST"], ...),
    finer = function(...) log(loglevels["FINER"], ...),
    fine = function(...) log(loglevels["FINE"], ...),
    debug = function(...) log(loglevels["DEBUG"], ...),
    info = function(...) log(loglevels["INFO"], ...),
    warn = function(...) log(loglevels["WARN"], ...),
    error = function(...) log(loglevels["ERROR"], ...)
    ) # methods
  ) # setRefClass

##
## this is part of the logging package. the logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with the nens libraray.  If not, see http://www.gnu.org/licenses/.
##
## Copyright (c) 2009..2013 by Mario Frasca
##

#'
#' Changes settings of logger or handler.
#'
#' @param container a logger, its name or a handler.
#' @param ... options to set for the container.
#'
#' @export
#'
updateOptions <- function(container, ...)
  UseMethod("updateOptions")

#' @describeIn updateOptions Update options for logger identified
#'   by name.
#' @export
#'
updateOptions.character <- function(container, ...) {
  ## container is really just the name of the container
  updateOptions(getLogger(container), ...)
}

#' @describeIn updateOptions Update options of logger or handler
#'   passed by reference.
#' @export
#'
updateOptions.environment <- function(container, ...) {
  ## the container is a logger
  config <- list(...)
  if ("level" %in% names(config)) {
    config$level <- namedLevel(config$level)
  } else if (get("name", container) == "") {
    # root logger
    config$level <- loglevels["INFO"]
  } else {
    config$level <- loglevels["NOTSET"]
  }

  for (key in names(config)) {
    if (key != "") {
      assign(key, config[[key]], container)
    }
  }
  invisible()
}

#' @describeIn updateOptions Update options of logger or handler
#'   passed by reference.
#' @export
#'
updateOptions.Logger <- function(container, ...) {
  updateOptions.environment(container, ...)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logging-entrypoints}
\alias{logging-entrypoints}
\alias{logdebug}
\alias{logfinest}
\alias{logfiner}
\alias{logfine}
\alias{loginfo}
\alias{logwarn}
\alias{logerror}
\alias{levellog}
\title{Entry points for logging actions}
\usage{
logdebug(msg, ..., logger = "")

logfinest(msg, ..., logger = "")

logfiner(msg, ..., logger = "")

logfine(msg, ..., logger = "")

loginfo(msg, ..., logger = "")

logwarn(msg, ..., logger = "")

logerror(msg, ..., logger = "")

levellog(level, msg, ..., logger = "")
}
\arguments{
\item{msg}{the textual message to be output, or the format for the \dots
arguments}

\item{...}{if present, msg is interpreted as a format and the \dots values
are passed to it to form the actual message.}

\item{logger}{the name of the logger to which we pass the record}

\item{level}{The logging level}
}
\description{
Generate a log record and pass it to the logging system.\cr
}
\details{
A log record gets timestamped and will be independently formatted by each
of the handlers handling it.\cr

Leading and trailing whitespace is stripped from the final message.
}
\examples{
logReset()
addHandler(writeToConsole)
loginfo('this goes to console')
logdebug('this stays silent')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{inbuilt-actions}
\alias{inbuilt-actions}
\alias{writeToConsole}
\alias{writeToFile}
\title{Predefined(sample) handler actions}
\usage{
writeToConsole(msg, handler, ...)

writeToFile(msg, handler, ...)
}
\arguments{
\item{msg}{A formated message to handle.}

\item{handler}{The handler environment containing its options. You can
register the same action to handlers with different properties.}

\item{...}{parameters provided by logger system to interact with the action.}
}
\description{
When you define a handler, you specify its name and the associated action.
A few predefined actions described below are provided.
}
\details{
A handler action is a function that accepts a formated message and handler
configuration.

Messages passed are filtered already regarding loglevel.

\dots parameters are used by logging system to interact with the action. \dots can
contain \var{dry} key to inform action that it meant to initialize itself. In the case
action should return TRUE if initialization succeded.

If it's not a dry run \dots contain the whole preformated \var{logging.record}.
A \var{logging.record} is a named list and has following structure:
\describe{
  \item{msg}{contains the real formatted message}
  \item{level}{message level as numeric}
  \item{levelname}{message level name}
  \item{logger}{name of the logger that generated it}
  \item{timestamp}{formatted message timestamp}
}

\code{writeToConsole} detects if crayon package is available and uses it
to color messages. The coloring can be switched off by means of configuring
the handler with \var{color_output} option set to FALSE.

\code{writeToFile} action expects file path to write to under
 \var{file} key in handler options.
}
\examples{
## define your own function and register it with a handler.
## author is planning a sentry client function.  please send
## any interesting function you may have written!

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{setMsgComposer}
\alias{setMsgComposer}
\title{Sets message composer for logger.}
\usage{
setMsgComposer(composer_f, container = "")
}
\arguments{
\item{composer_f}{message composer function (type: function(msg, ...))}

\item{container}{name of logger to reser message composer for (type: character)}
}
\description{
Message composer is used to compose log message out of formating string and arguments.
It is function with signature \code{function(msg, ...)}. Formating message is passed under msg
and formating arguments are passed as \code{...}.
}
\details{
If message composer is not set default is in use (realized with \code{sprintf}). If message
composer is not set for sub-logger, parent's message composer will be used.
}
\examples{
setMsgComposer(function(msg, ...) paste0("s-", msg, "-e"))
loginfo("a message") # will log '<TS> INFO::s-a message-e'
resetMsgComposer()
loginfo("a message") # will log '<TS> INFO::a message'

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/updateOptions.R
\name{updateOptions}
\alias{updateOptions}
\alias{updateOptions.character}
\alias{updateOptions.environment}
\alias{updateOptions.Logger}
\title{Changes settings of logger or handler.}
\usage{
updateOptions(container, ...)

\method{updateOptions}{character}(container, ...)

\method{updateOptions}{environment}(container, ...)

\method{updateOptions}{Logger}(container, ...)
}
\arguments{
\item{container}{a logger, its name or a handler.}

\item{...}{options to set for the container.}
}
\description{
Changes settings of logger or handler.
}
\section{Methods (by class)}{
\itemize{
\item \code{character}: Update options for logger identified
by name.

\item \code{environment}: Update options of logger or handler
passed by reference.

\item \code{Logger}: Update options of logger or handler
passed by reference.
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/namedLevels.R
\docType{data}
\name{loglevels}
\alias{loglevels}
\title{The logging levels, names and values}
\format{An object of class \code{numeric} of length 11.}
\usage{
loglevels
}
\description{
This list associates names to values and vice versa.\cr
Names and values are the same as in the python standard logging module.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{setLevel}
\alias{setLevel}
\title{Set \var{logging.level} for the object.}
\usage{
setLevel(level, container = "")
}
\arguments{
\item{level}{The new level for this object. Can be numeric or character.}

\item{container}{a logger, its name or a handler. Default is root logger.}
}
\description{
Alter an existing logger or handler, setting its \var{logging.level} to a new
value. You can access loggers by name, while you must use \code{getHandler} to
get a handler.
}
\examples{
basicConfig()
setLevel("FINEST")
setLevel("DEBUG", getHandler("basic.stdout"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{logging-package}
\alias{logging-package}
\title{A tentative logging package.}
\description{
A logging package emulating the Python logging package.

What you find here behaves similarly to what you also find in Python.
This includes hierarchic loggers, multiple handlers at each logger,
the possibility to specify a formatter for each handler (one default
formatter is given), same levels (names and numeric values) as Python's
logging package, a simple logging.BasicConfig function to quickly put
yourself in a usable situation...

This package owes a lot to my employer, r-forge, the stackoverflow community,
Brian Lee Yung Rowe's futile package (v1.1) and the documentation of
the Python logging package.
}
\details{
Index:
\describe{
\item{\code{basicConfig}}{bootstrapping the logging package}
\item{\code{addHandler}}{add a handler to a logger}
\item{\code{getLogger}}{set defaults and get current values for a logger}
\item{\code{removeHandler}}{remove a handler from a logger}
\item{\code{setLevel}}{set logging.level for a logger}
}

To use this package, include logging instructions in your code, possibly
like this:

\code{library(logging)}\cr
\code{basicConfig()}\cr
\code{addHandler(writeToFile, logger="company", file="sample.log")}\cr
\code{loginfo("hello world", logger="")}\cr
\code{logwarn("hello company", logger="company.module")}

The \code{basicConfig} function adds a console handler to the root logger,
while the explicitly called \code{addHandler} adds a file handler to the
'company' logger. the effect of the above example is two lines on the
console and just one line in the \code{sample.log} file.

The web pages for this package on r-forge are kept decently up to date
and contain a usable tutorial. Check the references.
}
\references{
the python logging module: \url{http://docs.python.org/library/logging.html}\cr
this package at github: \url{https://github.com/WLOGSolutions/r-logging/}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{resetMsgComposer}
\alias{resetMsgComposer}
\title{Resets previously set message composer.}
\usage{
resetMsgComposer(container = "")
}
\arguments{
\item{container}{name of logger to reser message composer for (type: character)}
}
\description{
Resets previously set message composer.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{bootstrapping}
\alias{bootstrapping}
\alias{basicConfig}
\alias{logReset}
\title{Bootstrapping the logging package.}
\usage{
basicConfig(level = 20)

logReset()
}
\arguments{
\item{level}{The logging level of the root logger. Defaults to INFO. Please do notice that
this has no effect on the handling level of the handler that basicConfig attaches to the
root logger.}
}
\description{
\code{basicConfig} and \code{logReset} provide a way to put the logging package
in a know initial state.
}
\details{
\code{basicConfig} creates the root logger, attaches a console handler(by
\var{basic.stdout} name) to it and sets the level of the handler to
\code{level}. You must not call \code{basicConfig} to for logger to work any more:
then root logger is created it gets initialized by default the same way as
\code{basicConfig} does. If you need clear logger to fill with you own handlers
use \code{logReset} to remove all default handlers.

\code{logReset} reinitializes the whole logging system as if the package had just been
loaded except it also removes all default handlers. Typically, you would want to call
\code{basicConfig} immediately after a call to \code{logReset}.
}
\examples{
basicConfig()
logdebug("not shown, basic is INFO")
logwarn("shown and timestamped")
logReset()
logwarn("not shown, as no handlers are present after a reset")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{getLogger}
\alias{getLogger}
\title{Set defaults and get the named logger.}
\usage{
getLogger(name = "", ...)
}
\arguments{
\item{name}{The name of the logger}

\item{...}{Any properties you may want to set in the newly created
logger. These have no effect if the logger is already present.}
}
\value{
The logger retrieved or registered.
}
\description{
Make sure a logger with a specific name exists and return it as a
\var{Logger} S4 object.  if not yet present, the logger will be created and
given the values specified in the \dots arguments.
}
\examples{
getLogger()  # returns the root logger
getLogger('test.sub')  # constructs a new logger and returns it
getLogger('test.sub')  # returns it again

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{handlers-management}
\alias{handlers-management}
\alias{addHandler}
\alias{removeHandler}
\title{Add a handler to or remove one from a logger.}
\usage{
addHandler(handler, ..., logger = "")

removeHandler(handler, logger = "")
}
\arguments{
\item{handler}{The name of the handler, or its action}

\item{...}{Extra parameters, to be stored in the handler list

\dots may contain extra parameters that will be passed to the handler
action. Some elements in the \dots will be interpreted here.}

\item{logger}{the name of the logger to which to attach the new handler,
defaults to the root logger.}
}
\description{
Use this function to maintain the list of handlers attached to a logger.\cr
\cr
\code{addHandler} and \code{removeHandler} are also offered as methods of the
\var{Logger} S4 class.
}
\details{
Handlers are implemented as environments. Within a logger a handler is
identified by its \var{name} and all handlers define at least the
three variables:
\describe{
  \item{level}{all records at level lower than this are skipped.}
  \item{formatter}{a function getting a record and returning a string}
  \item{\code{action(msg, handler)}}{a function accepting two parameters: a
     formatted log record and the handler itself. making the handler a
     parameter of the action allows us to have reusable action functions.}
}

Being an environment, a handler may define as many variables as you
think you need.  keep in mind the handler is passed to the action
function, which can check for existence and can use all variables that
the handler defines.
}
\examples{
logReset()
addHandler(writeToConsole)
names(getLogger()[["handlers"]])
loginfo("test")
removeHandler("writeToConsole")
logwarn("test")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{getHandler}
\alias{getHandler}
\title{Retrieves a handler from a logger.}
\usage{
getHandler(handler, logger = "")
}
\arguments{
\item{handler}{The name of the handler, or its action.}

\item{logger}{Optional: the name of the logger. Defaults to the root logger.}
}
\value{
The retrieved handler object. It returns NULL if handler is not registerd.
}
\description{
Handlers are not uniquely identified by their name. Only within the logger to which
they are attached is their name unique. This function is here to allow you grab a
handler from a logger so you can examine and alter it.

Typical use of this function is in \code{setLevel(newLevel, getHandler(...))}.
}
\examples{
logReset()
addHandler(writeToConsole)
getHandler("basic.stdout")

}
